@@ -20,7 +20,6 @@ class _select_grammeme_from(object):
     """
     def __init__(self, grammeme_set):
         self.grammeme_set = grammeme_set
-
         # ... are descriptors not magical enough?
 
         # In order to fight typos, raise an exception
@@ -33,7 +32,8 @@ class _select_grammeme_from(object):
                 if other is None:
                     return False
                 if other not in grammeme_set:
-                    raise ValueError("'%s' is not a valid grammeme for this attribute." % other)
+                    known_grammemes = ", ".join(grammeme_set)
+                    raise ValueError("'%s' is not a valid grammeme for this attribute. Valid grammemes: %s" % (other, known_grammemes))
                 return _str.__eq__(self, other)
 
             def __ne__(self, other):
@@ -73,7 +73,7 @@ class OpencorporaTag(object):
 
         >>> from pymorphy2 import MorphAnalyzer
         >>> morph = MorphAnalyzer()
-        >>> Tag = morph.TagClass # get an initialzed Tag class
+        >>> Tag = morph.TagClass  # get an initialzed Tag class
         >>> tag = Tag('VERB,perf,tran plur,impr,excl')
         >>> tag
         OpencorporaTag('VERB,perf,tran plur,impr,excl')
@@ -120,12 +120,10 @@ class OpencorporaTag(object):
         >>> tag.POS == 'plur'
         Traceback (most recent call last):
         ...
-        ValueError: 'plur' is not a valid grammeme for this attribute.
+        ValueError: 'plur' is not a valid grammeme for this attribute. Valid grammemes: ...
 
     """
 
-    __slots__ = ['_grammemes_tuple', '_grammemes_cache', '_str', '_POS']
-
     # Grammeme categories
     # (see http://opencorpora.org/dict.php?act=gram for a full set)
     # -------------------------------------------------------------
@@ -240,6 +238,8 @@ class OpencorporaTag(object):
     }
     _GRAMMEME_INDICES = collections.defaultdict(int)
     _GRAMMEME_INCOMPATIBLE = collections.defaultdict(set)
+    _LAT2CYR = None
+    _CYR2LAT = None
     KNOWN_GRAMMEMES = set()
 
     _NUMERAL_AGREEMENT_GRAMMEMES = (
@@ -257,6 +257,9 @@ class OpencorporaTag(object):
         'voct': 'nomn'
     }
 
+    __slots__ = ['_grammemes_tuple', '_grammemes_cache', '_str', '_POS',
+                 '_cyr', '_cyr_grammemes_cache']
+
     def __init__(self, tag):
         self._str = tag
         # XXX: we loose information about which grammemes
@@ -275,13 +278,8 @@ class OpencorporaTag(object):
         self._grammemes_tuple = grammemes_tuple
         self._POS = self._grammemes_tuple[0]
         self._grammemes_cache = None
-
-    @property
-    def grammemes(self):
-        """ A frozenset with grammemes for this tag. """
-        if self._grammemes_cache is None:
-            self._grammemes_cache = frozenset(self._grammemes_tuple)
-        return self._grammemes_cache
+        self._cyr_grammemes_cache = None
+        self._cyr = None
 
     # attributes for grammeme categories
     POS = _select_grammeme_from(PARTS_OF_SPEECH)
@@ -297,6 +295,38 @@ class OpencorporaTag(object):
     transitivity = _select_grammeme_from(TRANSITIVITY)
     voice = _select_grammeme_from(VOICES)
 
+    @property
+    def grammemes(self):
+        """ A frozenset with grammemes for this tag. """
+        if self._grammemes_cache is None:
+            self._grammemes_cache = frozenset(self._grammemes_tuple)
+        return self._grammemes_cache
+
+    @property
+    def grammemes_cyr(self):
+        """ A frozenset with Cyrillic grammemes for this tag. """
+        if self._cyr_grammemes_cache is None:
+            cyr_grammemes = [self._LAT2CYR[g] for g in self._grammemes_tuple]
+            self._cyr_grammemes_cache = frozenset(cyr_grammemes)
+        return self._cyr_grammemes_cache
+
+    @property
+    def cyr_repr(self):
+        """ Cyrillic representation of this tag """
+        if self._cyr is None:
+            self._cyr = self.lat2cyr(self)
+        return self._cyr
+
+    @classmethod
+    def cyr2lat(cls, tag_or_grammeme):
+        """ Return Latin representation for ``tag_or_grammeme`` string """
+        return _translate_tag(tag_or_grammeme, cls._CYR2LAT)
+
+    @classmethod
+    def lat2cyr(cls, tag_or_grammeme):
+        """ Return Cyrillic representation for ``tag_or_grammeme`` string """
+        return _translate_tag(tag_or_grammeme, cls._LAT2CYR)
+
     def __contains__(self, grammeme):
 
         # {'NOUN', 'sing'} in tag
@@ -387,7 +417,13 @@ class OpencorporaTag(object):
         """
         Replace rare cases (loc2/voct/...) with common ones (loct/nomn/...).
         """
-        return frozenset(cls.RARE_CASES.get(g,g) for g in grammemes)
+        return frozenset(cls.RARE_CASES.get(g, g) for g in grammemes)
+
+    @classmethod
+    def add_grammemes_to_known(cls, lat, cyr):
+        cls.KNOWN_GRAMMEMES.add(lat)
+        cls._LAT2CYR[lat] = cyr
+        cls._CYR2LAT[cyr] = lat
 
     @classmethod
     def _init_grammemes(cls, dict_grammemes):
@@ -403,18 +439,22 @@ class OpencorporaTag(object):
             ]
 
         """
-        gr = dict((name, parent) for (name, parent, alias, description) in dict_grammemes)
-
-        # figure out parents & children
-        children = collections.defaultdict(set)
-        for index, (name, parent, alias, description) in enumerate(dict_grammemes):
-            if parent:
-                children[parent].add(name)
-            if gr.get(parent, None): # parent's parent
-                children[gr[parent]].add(name)
-
         with threading.RLock():
-            cls.KNOWN_GRAMMEMES = set(gr.keys())
+            cls.KNOWN_GRAMMEMES = set()
+            cls._CYR2LAT = {}
+            cls._LAT2CYR = {}
+            for name, parent, alias, description in dict_grammemes:
+                cls.add_grammemes_to_known(name, alias)
+
+            gr = dict((name, parent) for (name, parent, alias, description) in dict_grammemes)
+
+            # figure out parents & children
+            children = collections.defaultdict(set)
+            for index, (name, parent, alias, description) in enumerate(dict_grammemes):
+                if parent:
+                    children[parent].add(name)
+                if gr.get(parent, None):  # parent's parent
+                    children[gr[parent]].add(name)
 
             # expand EXTRA_INCOMPATIBLE
             for grammeme, g_set in cls._EXTRA_INCOMPATIBLE.items():
@@ -533,6 +573,26 @@ class CyrillicOpencorporaTag(OpencorporaTag):
         for name, parent, alias, description in dict_grammemes:
             cls._GRAMMEME_ALIAS_MAP[name] = alias
 
+
+def _translate_tag(tag, mapping):
+    """
+    Translate ``tag`` string according to ``mapping``, assuming grammemes
+    are separated by commas or whitespaces. Commas/whitespaces positions
+    are preserved.
+    """
+    if isinstance(tag, OpencorporaTag):
+        tag = str(tag)
+    return " ".join([
+        _translate_comma_separated(whitespace_separated_part, mapping)
+        for whitespace_separated_part in tag.split()
+    ])
+
+
+def _translate_comma_separated(tag_part, mapping):
+    grammemes = [mapping.get(tok, tok) for tok in tag_part.split(',')]
+    return ",".join(grammemes)
+
+
 registry = dict()
 
 for tag_type in [CyrillicOpencorporaTag, OpencorporaTag]:
